//
//  AssetManager.js
//  Kite
//
//  Created by Joe Rhodes
//  Copyright (c) 2015 Joe Rhodes. All rights reserved.
//

AssetManager = function(docURL, sketchDoc) {

    this.docURL = docURL
    this.sketchDoc = sketchDoc
    this.contentIdentifiers = []
}

AssetManager.prototype = {

    saveLayerAsset: function(layer, sketchPage) {

        var imageURL = layer.urlForImageOfLayerInDocument(this.sketchDoc, sketchPage)
        var sanitizedLayerName = this.sanitizedLayerName(layer.name)
        var contentIdentifier = this.contentIdentifierForLayerName(sanitizedLayerName)
        var destinationURL = this.imagePathForContentIdentifier(contentIdentifier)

        if (imageURL && sanitizedLayerName.length > 0 && contentIdentifier.length > 0)
        {
            var fileManager = NSFileManager.defaultManager()
            var imageMovedSuccessfully = [fileManager moveItemAtURL:imageURL toURL:destinationURL error:null]

            if (imageMovedSuccessfully == false) {
                log("Failed to write png data for contentIdentifier: " + contentIdentifier)
            }
        }

        return destinationURL.lastPathComponent()
    },

    save: function(exportInfo) {
        log("AssetManager Save!")

        if (this.createAssetsDirectory() == false) {
            return false
        }

        return true
    },

    // -------------------------------------------------------------------------------------------
    // Helper
    // -------------------------------------------------------------------------------------------

    imagePathForContentIdentifier: function(contentIdentifier) {
        var imageURL = this.assetsDirectoryURL().URLByAppendingPathComponent(contentIdentifier)

        return imageURL.URLByAppendingPathExtension("png")
    },

    contentIdentifierForLayerName: function(layerName, increment) {

        var newLayerName = layerName

        if (increment) {
            newLayerName = layerName + "-" + increment
        }
        else {
            increment = 0
        }

        if (this.contentIdentifiers.indexOf(newLayerName) == -1) {

            this.contentIdentifiers.push(newLayerName)
            return newLayerName
        }
        else {
            return this.contentIdentifierForLayerName(layerName, ++increment)
        }
    },

    sanitizedLayerName: function(layerName) {
        // Replace all whitespace with a dash
        //
        layerName = layerName.trim().replace(/\s+/ig, "-").trim()

        return layerName
    },

    createAssetsDirectory: function() {
        var fileManager = NSFileManager.defaultManager()
        var assetsDirectoryURL = this.assetsDirectoryURL()

        // Does it already exist ?
        //
        if (fileManager.fileExistsAtPath( assetsDirectoryURL.path() )) {
            return true
        }

        var error = null
        var successful = [fileManager createDirectoryAtURL:assetsDirectoryURL withIntermediateDirectories:true attributes:null error:error]

        if (successful == false) {
            log("Failed to create /Assets directory at path!! : " + assetsDirectoryURL.path())
            return false
        }

        return true
    },

    assetsDirectoryURL: function() {
        return this.docURL.URLByAppendingPathComponent("Assets")
    }
}